using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

//using XTreme.Common;
//using Library.Logger;
using Syncfusion.Windows.Forms.Grid;

namespace SelectionsApp
{
	/// <summary>
	/// ExecutionListingForm displays Live Executions
	/// </summary>
	public class ExecutionListingForm : System.Windows.Forms.Form
	{

		#region Variables

		private System.ComponentModel.IContainer components;

		private long key = 0;
		private bool bGroup = false;
		private bool bMatchedOnly;// = true;
		private bool bMultiSelect;// = false;		
		private bool bUnreportedOnly;// = false;
		private bool bCheckExpansion = true;
		private bool bCheckSelections = true;
		
		private Hashtable pkHash, sortHash, expansionHash; // to be used for filter, sort, expansion

		private GridRangeInfoList gms_M, gms_U;
		private int val_M = 0, val_U = 0;

		protected Timer flashTimer;
		protected int flashCount, flashCountMax;
		private int topRow_M = 1, topRow_U = 1;
		//		protected bool bFlashing = false;

//		public static XTreme.Common.FormConfig FormConfig = new XTreme.Common.FormConfig();

		private DataSet setExecutions;
		private GridHierarchyLevel hlr = null;
		private DataView orderView, execView;
		private DataTable orderTable, execTable;
		private GridModelDataBinder gridBinder = null;
		private Hashtable jargonsHash, settingsHash, colorsHash;
        	
		private System.Windows.Forms.TabPage tabMatch;
		private System.Windows.Forms.TabPage tabUnmatch;
		private System.Windows.Forms.Label labelStatus;
		private System.Windows.Forms.ToolTip toolTipExec;
		private System.Windows.Forms.CheckBox checkBoxGroup;
		private System.Windows.Forms.TabControl tabExecutions;
		private Syncfusion.Windows.Forms.Grid.ExtendedDataBoundGrid edbgMain;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn gridBoundColumn1;
		private System.Windows.Forms.Button btnReConnect;
		private System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.StatusBarPanel statusBarPanel1;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn ColAction;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn ColQuantity;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn ColSymbol;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn ColPrice;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn ColMarket;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn ColReported;
		private Syncfusion.Windows.Forms.Grid.GridBoundColumn colOIRN;
		private System.Windows.Forms.Button btnConfigure;
		private System.Windows.Forms.CheckBox checkBoxUnreportedOnly;

		#endregion

		#region Events And Delegates

		protected delegate void ExecutionInvoker(Hashtable execution);
		protected delegate void ExecutionsInvoker(IList executions);
		protected delegate void stringInvoker(string text);
		protected delegate void StatusInvoker(string strValue);
		protected delegate void TabInvoker(bool bShow);
		protected delegate void IntInvoker(int val);
		protected delegate IList ExecInvoker();

		/// <summary>
		/// OnRowDoubleClick is fired when a user double clicks on the Grid
		/// </summary>
		public event EventHandler OnRowDoubleClick;

		/// <summary>
		/// OnReconnectClick is fired when a user clicks the reconnect button
		/// </summary>
		public event EventHandler OnReconnectClick;

		/// <summary>
		/// OnConfigure is fired when a user clicks the Configure button
		/// </summary>
		public event EventHandler OnConfigure;

		#endregion

		#region Constructor & Dispose

		public ExecutionListingForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.edbgMain.Binder.ItemChanged += new ItemChangedEventHandler(this.OnItemChange);
			this.edbgMain.Model.SelectionChanged += new GridSelectionChangedEventHandler(this.OnGridSelChanged);

			this.gms_M = new GridRangeInfoList();
			this.gms_U = new GridRangeInfoList();

			//			this.gms_M.Clear();
			//			this.gms_U.Clear();
					
			Setup();

			this.GroupExecution = false;
			this.GroupExecution = true;
			this.MatchedOnly = true;
			this.MultiSelect = false;
			this.checkBoxUnreportedOnly.Checked = false;
			this.EnableReConnect = false;

			flashTimer = new Timer();
			flashTimer.Tick += new EventHandler(this.FlashDialog);
			flashTimer.Interval = 300;
			flashCountMax = 3;

			pkHash = new Hashtable();
			pkHash[Nodes.ORDERID] = "Order_ID";
			pkHash[Nodes.SEQNUM] = "SeqNum";
			pkHash[Nodes.Action] = "Action";
			pkHash[Nodes.SYMBOL] = "Symbol";
			pkHash[Nodes.Market] = "Market";

			expansionHash = new Hashtable();

			sortHash = new Hashtable();
			foreach(GridBoundColumn gbc in this.edbgMain.GridBoundColumns)
			{
				sortHash[gbc.MappingName] = false;
			}
			
			this.edbgMain.SetColWidth(2,8, new int[] { 45,50,50,40,45,75,110 });
			this.edbgMain.SetRowHeight(0,0,18);
			InitializeLP();
			//this.edbgMain.SetColWidth(7,7,75);
			//this.edbgMain.SetColWidth(8,8,110);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
			Initialize();
		}
		protected void Initialize()
		{
//			FormConfig.MonitorForm(this);
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle1 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle2 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle3 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridBaseStyle gridBaseStyle4 = new Syncfusion.Windows.Forms.Grid.GridBaseStyle();
			Syncfusion.Windows.Forms.Grid.GridStyleInfo gridStyleInfo1 = new Syncfusion.Windows.Forms.Grid.GridStyleInfo();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ExecutionListingForm));
			this.tabExecutions = new System.Windows.Forms.TabControl();
			this.tabMatch = new System.Windows.Forms.TabPage();
			this.tabUnmatch = new System.Windows.Forms.TabPage();
			this.edbgMain = new Syncfusion.Windows.Forms.Grid.ExtendedDataBoundGrid();
			this.ColAction = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.ColQuantity = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.ColSymbol = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.ColPrice = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.ColMarket = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.ColReported = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.colOIRN = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.gridBoundColumn1 = new Syncfusion.Windows.Forms.Grid.GridBoundColumn();
			this.labelStatus = new System.Windows.Forms.Label();
			this.toolTipExec = new System.Windows.Forms.ToolTip(this.components);
			this.checkBoxGroup = new System.Windows.Forms.CheckBox();
			this.checkBoxUnreportedOnly = new System.Windows.Forms.CheckBox();
			this.btnReConnect = new System.Windows.Forms.Button();
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.statusBarPanel1 = new System.Windows.Forms.StatusBarPanel();
			this.btnConfigure = new System.Windows.Forms.Button();
			this.tabExecutions.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.edbgMain)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).BeginInit();
			this.SuspendLayout();
			// 
			// tabExecutions
			// 
			this.tabExecutions.Anchor = ((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.tabExecutions.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.tabMatch,
																						this.tabUnmatch});
			this.tabExecutions.Name = "tabExecutions";
			this.tabExecutions.SelectedIndex = 0;
			this.tabExecutions.Size = new System.Drawing.Size(492, 21);
			this.tabExecutions.TabIndex = 0;
			this.tabExecutions.SelectedIndexChanged += new System.EventHandler(this.OnTabChanged);
			// 
			// tabMatch
			// 
			this.tabMatch.Location = new System.Drawing.Point(4, 22);
			this.tabMatch.Name = "tabMatch";
			this.tabMatch.Size = new System.Drawing.Size(484, 0);
			this.tabMatch.TabIndex = 0;
			this.tabMatch.Text = "Matched";
			// 
			// tabUnmatch
			// 
			this.tabUnmatch.Location = new System.Drawing.Point(4, 22);
			this.tabUnmatch.Name = "tabUnmatch";
			this.tabUnmatch.Size = new System.Drawing.Size(484, -5);
			this.tabUnmatch.TabIndex = 1;
			this.tabUnmatch.Text = "UnMatched";
			// 
			// edbgMain
			// 
			this.edbgMain.AllowDragSelectedCols = true;
			this.edbgMain.AllowResizeToFit = false;
			this.edbgMain.AllowSelection = (Syncfusion.Windows.Forms.Grid.GridSelectionFlags.Row | Syncfusion.Windows.Forms.Grid.GridSelectionFlags.AlphaBlend);
			this.edbgMain.AlphaBlendSelectionColor = System.Drawing.Color.FromArgb(((System.Byte)(1)), ((System.Byte)(0)), ((System.Byte)(0)), ((System.Byte)(0)));
			this.edbgMain.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			gridBaseStyle1.Name = "Header";
			gridBaseStyle1.StyleInfo.Borders.Bottom = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.Borders.Left = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.Borders.Right = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.Borders.Top = new Syncfusion.Windows.Forms.Grid.GridBorder(Syncfusion.Windows.Forms.Grid.GridBorderStyle.None);
			gridBaseStyle1.StyleInfo.CellType = "Header";
			gridBaseStyle1.StyleInfo.Font.Bold = true;
			gridBaseStyle1.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.Control);
			gridBaseStyle1.StyleInfo.VerticalAlignment = Syncfusion.Windows.Forms.Grid.GridVerticalAlignment.Middle;
			gridBaseStyle2.Name = "Column Header";
			gridBaseStyle2.StyleInfo.BaseStyle = "Header";
			gridBaseStyle2.StyleInfo.CellAppearance = Syncfusion.Windows.Forms.Grid.GridCellAppearance.Flat;
			gridBaseStyle2.StyleInfo.CellType = "ColumnHeaderCell";
			gridBaseStyle2.StyleInfo.Enabled = false;
			gridBaseStyle2.StyleInfo.Font.Bold = false;
			gridBaseStyle2.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			gridBaseStyle2.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			gridBaseStyle2.StyleInfo.WrapText = false;
			gridBaseStyle3.Name = "Row Header";
			gridBaseStyle3.StyleInfo.BaseStyle = "Header";
			gridBaseStyle3.StyleInfo.CellType = "RowHeaderCell";
			gridBaseStyle3.StyleInfo.Enabled = true;
			gridBaseStyle3.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Left;
			gridBaseStyle4.Name = "Standard";
			gridBaseStyle4.StyleInfo.CheckBoxOptions.CheckedValue = "True";
			gridBaseStyle4.StyleInfo.CheckBoxOptions.UncheckedValue = "False";
			gridBaseStyle4.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.Window);
			this.edbgMain.BaseStylesMap.AddRange(new Syncfusion.Windows.Forms.Grid.GridBaseStyle[] {
																									   gridBaseStyle1,
																									   gridBaseStyle2,
																									   gridBaseStyle3,
																									   gridBaseStyle4});
			this.edbgMain.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.edbgMain.EnableAddNew = false;
			this.edbgMain.EnableEdit = false;
			this.edbgMain.EnableRemove = false;
			this.edbgMain.GridBoundColumns.AddRange(new Syncfusion.Windows.Forms.Grid.GridBoundColumn[] {
																											this.ColAction,
																											this.ColQuantity,
																											this.ColSymbol,
																											this.ColPrice,
																											this.ColMarket,
																											this.ColReported,
																											this.colOIRN});
			this.edbgMain.HScrollBehavior = Syncfusion.Windows.Forms.Grid.GridScrollbarMode.Automatic;
			this.edbgMain.ListBoxSelectionMode = System.Windows.Forms.SelectionMode.One;
			this.edbgMain.Location = new System.Drawing.Point(0, 19);
			this.edbgMain.MinResizeColSize = 30;
			this.edbgMain.MinResizeRowSize = 5;
			this.edbgMain.Name = "edbgMain";
			this.edbgMain.Properties.ResizingCellsLinesColor = System.Drawing.Color.Black;
			this.edbgMain.Properties.RowHeaders = false;
			this.edbgMain.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.HideAlways;
			this.edbgMain.Size = new System.Drawing.Size(468, 289);
			this.edbgMain.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.None;
			this.edbgMain.TabIndex = 1;
			gridStyleInfo1.CellType = "Static";
			gridStyleInfo1.ReadOnly = true;
			this.edbgMain.TableStyle = gridStyleInfo1;
			this.edbgMain.VScrollBehavior = Syncfusion.Windows.Forms.Grid.GridScrollbarMode.Automatic;
			this.edbgMain.CellClick += new Syncfusion.Windows.Forms.Grid.GridCellClickEventHandler(this.OnGridCellClick);
			this.edbgMain.RowCollapsed += new Syncfusion.Windows.Forms.Grid.GridRowEventHandler(this.OnRowCollapsed);
			this.edbgMain.RowExpanded += new Syncfusion.Windows.Forms.Grid.GridRowEventHandler(this.OnRowExpanded);
			this.edbgMain.PrepareViewStyleInfo += new Syncfusion.Windows.Forms.Grid.GridPrepareViewStyleInfoEventHandler(this.onPrepareViewStyleInfo);
			this.edbgMain.RowDoubleClick += new Syncfusion.Windows.Forms.Grid.ExtendedDataBoundGrid.RowClickDelegate(this.OnDoubleClick);
			this.edbgMain.DrawCell += new Syncfusion.Windows.Forms.Grid.GridDrawCellEventHandler(this.OnDrawCell);
			// 
			// ColAction
			// 
			this.ColAction.MappingName = "Action";
			this.ColAction.ReadOnly = true;
			this.ColAction.StyleInfo.CellType = "Static";
			this.ColAction.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.Window);
			this.ColAction.StyleInfo.ReadOnly = true;
			this.ColAction.StyleInfo.TextAlign = Syncfusion.Windows.Forms.Grid.GridTextAlign.Right;
			this.ColAction.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			this.ColAction.StyleInfo.WrapText = false;
			// 
			// ColQuantity
			// 
			this.ColQuantity.MappingName = "Quantity";
			this.ColQuantity.ReadOnly = true;
			this.ColQuantity.StyleInfo.CellType = "Static";
			this.ColQuantity.StyleInfo.CellValueType = typeof(System.UInt64);
			this.ColQuantity.StyleInfo.CurrencyEdit.CurrencyDecimalDigits = 2;
			this.ColQuantity.StyleInfo.CurrencyEdit.CurrencySymbol = "$";
			this.ColQuantity.StyleInfo.Format = "#,###,###";
			this.ColQuantity.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			this.ColQuantity.StyleInfo.Interior = new Syncfusion.Drawing.BrushInfo(System.Drawing.SystemColors.Window);
			this.ColQuantity.StyleInfo.ReadOnly = true;
			this.ColQuantity.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			this.ColQuantity.StyleInfo.WrapText = false;
			// 
			// ColSymbol
			// 
			this.ColSymbol.MappingName = "Symbol";
			this.ColSymbol.ReadOnly = true;
			this.ColSymbol.StyleInfo.CellType = "Static";
			this.ColSymbol.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			this.ColSymbol.StyleInfo.ReadOnly = true;
			this.ColSymbol.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			this.ColSymbol.StyleInfo.WrapText = false;
			// 
			// ColPrice
			// 
			this.ColPrice.MappingName = "Price";
			this.ColPrice.ReadOnly = true;
			this.ColPrice.StyleInfo.CellType = "Static";
			this.ColPrice.StyleInfo.CellValueType = typeof(System.Double);
			this.ColPrice.StyleInfo.CurrencyEdit.CurrencyDecimalDigits = 4;
			this.ColPrice.StyleInfo.Format = "0.0000";
			this.ColPrice.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			this.ColPrice.StyleInfo.ReadOnly = true;
			this.ColPrice.StyleInfo.TextAlign = Syncfusion.Windows.Forms.Grid.GridTextAlign.Default;
			this.ColPrice.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			this.ColPrice.StyleInfo.WrapText = false;
			// 
			// ColMarket
			// 
			this.ColMarket.MappingName = "Market";
			this.ColMarket.ReadOnly = true;
			this.ColMarket.StyleInfo.CellType = "Static";
			this.ColMarket.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			this.ColMarket.StyleInfo.ReadOnly = true;
			this.ColMarket.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			this.ColMarket.StyleInfo.WrapText = false;
			// 
			// ColReported
			// 
			this.ColReported.HeaderText = "Reported On";
			this.ColReported.MappingName = "ReportedOn";
			this.ColReported.ReadOnly = true;
			this.ColReported.StyleInfo.CellType = "Static";
			this.ColReported.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			this.ColReported.StyleInfo.ReadOnly = true;
			this.ColReported.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			this.ColReported.StyleInfo.WrapText = false;
			// 
			// colOIRN
			// 
			this.colOIRN.HeaderText = "Order No./ Ref. No.";
			this.colOIRN.MappingName = "OIRN";
			this.colOIRN.ReadOnly = true;
			this.colOIRN.StyleInfo.CellType = "Static";
			this.colOIRN.StyleInfo.Format = "";
			this.colOIRN.StyleInfo.HorizontalAlignment = Syncfusion.Windows.Forms.Grid.GridHorizontalAlignment.Center;
			this.colOIRN.StyleInfo.MaskEdit.Mask = "";
			this.colOIRN.StyleInfo.ReadOnly = true;
			this.colOIRN.StyleInfo.Trimming = System.Drawing.StringTrimming.EllipsisCharacter;
			this.colOIRN.StyleInfo.WrapText = false;
			// 
			// gridBoundColumn1
			// 
			this.gridBoundColumn1.MappingName = "OrderID";
			this.gridBoundColumn1.StyleInfo.CellType = "Static";
			// 
			// labelStatus
			// 
			this.labelStatus.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.labelStatus.ForeColor = System.Drawing.Color.Red;
			this.labelStatus.Location = new System.Drawing.Point(200, 328);
			this.labelStatus.Name = "labelStatus";
			this.labelStatus.Size = new System.Drawing.Size(24, 16);
			this.labelStatus.TabIndex = 2;
			this.labelStatus.Visible = false;
			this.labelStatus.TextChanged += new System.EventHandler(this.OnStatusTextChanged);
			// 
			// checkBoxGroup
			// 
			this.checkBoxGroup.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left);
			this.checkBoxGroup.Checked = true;
			this.checkBoxGroup.CheckState = System.Windows.Forms.CheckState.Checked;
			this.checkBoxGroup.Location = new System.Drawing.Point(4, 320);
			this.checkBoxGroup.Name = "checkBoxGroup";
			this.checkBoxGroup.Size = new System.Drawing.Size(124, 16);
			this.checkBoxGroup.TabIndex = 3;
			this.checkBoxGroup.Text = "Group By Order No.";
			this.checkBoxGroup.CheckedChanged += new System.EventHandler(this.onGroupCheckChanged);
			// 
			// checkBoxUnreportedOnly
			// 
			this.checkBoxUnreportedOnly.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left);
			this.checkBoxUnreportedOnly.Location = new System.Drawing.Point(4, 348);
			this.checkBoxUnreportedOnly.Name = "checkBoxUnreportedOnly";
			this.checkBoxUnreportedOnly.Size = new System.Drawing.Size(108, 16);
			this.checkBoxUnreportedOnly.TabIndex = 4;
			this.checkBoxUnreportedOnly.Text = "Unreported Only";
			this.checkBoxUnreportedOnly.CheckedChanged += new System.EventHandler(this.OnUnreportedCheckChanged);
			// 
			// btnReConnect
			// 
			this.btnReConnect.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btnReConnect.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnReConnect.Location = new System.Drawing.Point(392, 312);
			this.btnReConnect.Name = "btnReConnect";
			this.btnReConnect.Size = new System.Drawing.Size(72, 24);
			this.btnReConnect.TabIndex = 5;
			this.btnReConnect.Text = "Reconnect";
			this.btnReConnect.Click += new System.EventHandler(this.OnReConnectClick);
			// 
			// statusBar1
			// 
			this.statusBar1.Location = new System.Drawing.Point(0, 364);
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						  this.statusBarPanel1});
			this.statusBar1.ShowPanels = true;
			this.statusBar1.Size = new System.Drawing.Size(468, 18);
			this.statusBar1.SizingGrip = false;
			this.statusBar1.TabIndex = 6;
			// 
			// statusBarPanel1
			// 
			this.statusBarPanel1.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.statusBarPanel1.Width = 468;
			// 
			// btnConfigure
			// 
			this.btnConfigure.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right);
			this.btnConfigure.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.btnConfigure.Location = new System.Drawing.Point(392, 340);
			this.btnConfigure.Name = "btnConfigure";
			this.btnConfigure.Size = new System.Drawing.Size(72, 24);
			this.btnConfigure.TabIndex = 7;
			this.btnConfigure.Text = "Configure...";
			this.btnConfigure.Click += new System.EventHandler(this.onConfigureClick);
			// 
			// ExecutionListingForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(468, 382);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.btnConfigure,
																		  this.statusBar1,
																		  this.btnReConnect,
																		  this.checkBoxUnreportedOnly,
																		  this.checkBoxGroup,
																		  this.labelStatus,
																		  this.tabExecutions,
																		  this.edbgMain});
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Name = "ExecutionListingForm";
			this.Text = "External Fills";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.ExecutionListingForm_Closing);
			this.VisibleChanged += new System.EventHandler(this.OnFormVisibleChanged);
			this.tabExecutions.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.edbgMain)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.statusBarPanel1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		#region Public Properties

		/// <summary>
		/// Sets if whether the reconnect button is enabled or not
		/// </summary>
		public bool EnableReConnect
		{
			set
			{
				this.btnReConnect.Enabled = value;
			}
		}

		/// <summary>
		/// Sets if whether the configure button is visibled or not
		/// </summary>
		protected bool EnableConfigure
		{
			set
			{
				this.btnConfigure.Visible = value;
			}
		}

		/// <summary>
		/// Gets the list of currently displayed Executions
		/// </summary>
		public IList Executions
		{
			get
			{
				if(!this.InvokeRequired)
					return GetExecutions();
				else
					return this.Invoke(new ExecInvoker(GetExecutions)) as IList;
			}
		}

		/// <summary>
		/// Gets the list of currently displayed Matched Executions
		/// </summary>

		public IList MatchedExecutions
		{
			get
			{
				if(!this.InvokeRequired)
					return GetMatchedExecutions();
				else
					return this.Invoke(new ExecInvoker(GetMatchedExecutions)) as IList;
			}
		}

		/// <summary>
		/// Gets the list of currently displayed Unmatched Executions
		/// </summary>
		/// 
		public IList UnmatchedExecutions
		{
			get
			{
				if(!this.InvokeRequired)
					return GetUnmatchedExecutions();
				else
					return this.Invoke(new ExecInvoker(GetUnmatchedExecutions)) as IList;
			}
		}

		/// <summary>
		/// Gets the list of currently Selected Executions
		/// </summary>

		public IList SelectedExecutions
		{
			get
			{
				if(!this.InvokeRequired)
					return GetSelectedExecutions();
				else
					return this.Invoke(new ExecInvoker(GetSelectedExecutions)) as IList;
			}
		}

		/// <summary>
		/// Gets the List of SelectedOrders
		/// </summary>

		public IList SelectedOrders
		{
			get
			{
				if(!this.InvokeRequired)
					return GetSelectedOrders();
				else
					return this.Invoke(new ExecInvoker(GetSelectedOrders)) as IList;
			}
						 
		}

		/// <summary>
		/// Gets the Executions' Context Menu
		/// </summary>

		public override ContextMenu ContextMenu
		{
			get
			{
				return this.edbgMain.ContextMenu;
			}
			set
			{
				this.edbgMain.ContextMenu = value;
			}
		}

		/// <summary>
		/// Gets or Sets if the Executions are to be grouped
		/// </summary>

		public bool GroupExecution
		{
			get
			{
				return bGroup;				
			}
			set
			{
				if(value)
				{
					if(!this.InvokeRequired)
						GroupSafe();
					else
						this.Invoke(new MethodInvoker(GroupSafe));
				}
				else
				{
					if(!this.InvokeRequired)
						UnGroupSafe();
					else
						this.Invoke(new MethodInvoker(UnGroupSafe));
				}
				bGroup = value;
			}
		}

		/// <summary>
		/// Gets or Sets if only the Matched Executions are to be displayed
		/// </summary>
		public bool MatchedOnly
		{
			get
			{
				return bMatchedOnly;
			}
			set
			{
				if(!this.InvokeRequired)
					ShowTab(value);
				else
					this.Invoke(new TabInvoker(ShowTab), new object[] { value });

				bMatchedOnly = value;
				EnableConfigure = !value;
			}
		}

		/// <summary>
		/// Gets or Sets if only the Unreported Executions are to be displayed
		/// </summary>

		public bool UnreportedOnly
		{
			get
			{
				return bUnreportedOnly;
			}
			set
			{
				if(!this.InvokeRequired)
					SetUnreportedSafe(value);
				else
					this.Invoke(new TabInvoker(SetUnreportedSafe), new object[] {value});

				bUnreportedOnly = value;
				//				this.edbgMain.BeginUpdate();
				//				this.edbgMain.Refresh();
				//				this.edbgMain.EndUpdate();
			}
		}

		/// <summary>
		/// Gets or Sets if the "Matched" tab would be on top or not
		/// </summary>
		public bool MatchedExecutionsOnTop
		{
			get
			{
				return IsOnTop();
			}
			set
			{
				if(value)
				{
					if(!this.InvokeRequired)
						SetTop();
					else
						this.Invoke(new MethodInvoker(SetTop));
				}
				else
				{
					if(!this.InvokeRequired)
						SetBottom();
					else
						this.Invoke(new MethodInvoker(SetBottom));
				}
			}
		}

		/// <summary>
		/// Gets or Sets the Title of the ExecutionListingForm
		/// </summary>
		public string Title
		{
			get
			{
				return this.Text;
			}
			set
			{
				this.Text = value;
			}
		}

		/// <summary>
		/// Sets the Jargons for the ExecutionListingForm
		/// </summary>
		public Hashtable Jargons
		{
			set
			{
				jargonsHash = value;
			}
		}

		/// <summary>
		/// Gets or Sets the Settings for the ExecutionListingForm
		/// </summary>

		public Hashtable Settings
		{
			get
			{
				return settingsHash;
			}
			set
			{
				settingsHash = value;
				if(!this.InvokeRequired)
					SetSettings(value);
				else
					this.Invoke(new ExecutionInvoker(SetSettings), new object[] {value});
			}
		}

		/// <summary>
		/// Gets or Sets the Color Scheme to use for the ExecutionListingForm
		/// </summary>

		public Hashtable ColorScheme
		{
			get
			{
				return colorsHash;
			}
			set
			{
				colorsHash = value;
				if(!this.InvokeRequired)
					SetColorScheme(value);
				else
					this.Invoke(new ExecutionInvoker(SetColorScheme), new object[] {value});
			}
		}

		/// <summary>
		/// Gets or Sets the text for the Status Bar of the ExecutionListingForm
		/// </summary>

		public string Status
		{
			/*get
			{
				return labelStatus.Text;
			}*/
			set
			{
				if(!this.InvokeRequired)
					SetStatus(value);
				else
					this.Invoke(new StatusInvoker(SetStatus), new object[] {value});
			}
		}

		/// <summary>
		/// Sets the Error Text for the Form and Flashes the Form
		/// </summary>
		public string Error
		{
			set
			{
				if(InvokeRequired)
					Invoke(new stringInvoker(SetErrorSafe),new object[1]{value});
				else
					SetErrorSafe(value);
			}
		}

		/// <summary>
		/// Gets or Sets the interval of the Error Flash in milliseconds
		/// </summary>
		public int FlashInterval
		{
			get
			{
				return flashTimer.Interval;
			}
			set
			{
				flashTimer.Interval = value;
			}
		}

		/// <summary>
		/// Gets or Sets the Maximum Flash Count
		/// </summary>
		public int MaxFlashCount
		{
			get
			{
				return flashCountMax;
			}
			set
			{
				flashCountMax = value;
			}
		}

		/// <summary>
		/// Gets or Sets if Multiple Selections is possible or not
		/// </summary>

		public bool MultiSelect
		{
			get
			{
				return bMultiSelect;
			}
			set
			{
				bMultiSelect = value;
				if(bMultiSelect)
					this.edbgMain.ListBoxSelectionMode = SelectionMode.MultiExtended;
				else
					this.edbgMain.ListBoxSelectionMode = SelectionMode.One;
			}
		}

		public int LPIcon
		{
			set
			{
				if(this.InvokeRequired)
					this.Invoke(new IntInvoker(SetLPIconSafe), new object[1]{value});
				else
					SetLPIconSafe(value);
			}
		}
		#endregion

		#region Public Methods

		/// <summary>
		/// Adds a new Execution to the Execution Listing. If the Execution already exists, it will be updated.
		/// </summary>
		/// <param name="execution"></param>
		public void AddExecution(Hashtable execution)
		{
			//this.edbgMain.BeginUpdate();
			//this.edbgMain.Model.SuspendChangeEvents();

			if(!this.InvokeRequired)
				AddExecutionSafe2(execution);
			else
				this.Invoke(new ExecutionInvoker(AddExecutionSafe2), new object[] { execution });

			//this.edbgMain.Model.ResumeChangeEvents();
			//this.edbgMain.EndUpdate();
		}
	
		/// <summary>
		/// Adds a list of Executions to the Execution Listing. If one or more Execution already exist, they will be updated.
		/// </summary>
		/// <param name="executions"></param>
		public void AddExecutions(IList executions)
		{
			if(!this.InvokeRequired)
				AddExecutionsSafe2(executions);
			else
				this.Invoke(new ExecutionsInvoker(AddExecutionsSafe2), new object[] { executions });
		}

		/// <summary>
		/// Clears All Executions from the Execution Listing.
		/// </summary>
		public void ClearExecutions()
		{
			if(!this.InvokeRequired)
				ClearExecutionsSafe();
			else
				this.Invoke(new MethodInvoker(ClearExecutionsSafe));			
		}

		/// <summary>
		/// Deletes an existing Execution from the Execution Listing.
		/// </summary>
		/// <param name="execution"></param>
		public void DeleteExecution(Hashtable execution)
		{
			if(!this.InvokeRequired)
				DeleteExecutionSafe(execution);
			else
				this.Invoke(new ExecutionInvoker(DeleteExecutionSafe), new object[] { execution });
		}

		/// <summary>
		/// Deletes the list of Executions (which exist) from the Execution Listing.
		/// </summary>
		/// <param name="executions"></param>
		public void DeleteExecutions(IList executions)
		{
			if(!this.InvokeRequired)
				DeleteExecutionsSafe(executions);
			else
				this.Invoke(new ExecutionsInvoker(DeleteExecutionsSafe), new object[] { executions });
		}


		/// <summary>
		/// PointToRowCol()
		/// </summary>
		/// <param name="pt">Point</param>
		/// <param name="row"></param>
		/// <param name="col"></param>
		/// <returns></returns>
		public bool PointToRowCol(Point pt, out int row, out int col)
		{
			return edbgMain.PointToRowCol(edbgMain.PointToClient(pt),out row, out col);
		}

		#endregion

		#region Protected/Helper Methods

		protected void SetUnreportedSafe(bool bValue)
		{
			this.edbgMain.BeginUpdate();
			SetUnreported(bValue);
			DoExpansion();
			this.edbgMain.EndUpdate();
		}

		protected void SetUnreported(bool bValue)
		{
			//			this.edbgMain.BeginUpdate();

			DataView cView = this.edbgMain.DataSource as DataView;
			if(cView == orderTable.DefaultView)
			{
				if(bValue)
				{
					HideRows();

					orderTable.DefaultView.RowFilter = SetAR(orderTable.DefaultView.RowFilter);
					orderTable.DefaultView.DataViewManager.DataViewSettings[execTable.TableName].RowFilter = SetUR(orderTable.DefaultView.DataViewManager.DataViewSettings[execTable.TableName].RowFilter);
				}
				else
				{
					ReCalculateOrders();
					
					orderTable.DefaultView.RowFilter = RemAR(orderTable.DefaultView.RowFilter);
					orderTable.DefaultView.DataViewManager.DataViewSettings[execTable.TableName].RowFilter = RemUR(orderTable.DefaultView.DataViewManager.DataViewSettings[execTable.TableName].RowFilter);
				}

				//this.edbgMain.ExpandAll(); //	DoExpansion();
			}
			else
			{
				if(bValue)
					execTable.DefaultView.RowFilter = SetUR(execTable.DefaultView.RowFilter);
				else
					execTable.DefaultView.RowFilter = RemUR(execTable.DefaultView.RowFilter);
			}

			//		this.edbgMain.EndUpdate();			
		}

		protected void HideRows()
		{
			DataRow[] oRows = orderTable.Select("Len(ReportedOn) > 0 AND AllReported = 'false'");
			for(int x=0; x < oRows.Length; x++)
			{
				oRows[x]["ReportedOn"] = System.DBNull.Value;
			}
		}
		protected void ReCalculateOrders()
		{
			DataRow[] oRows = orderTable.Select("AllReported = 'false'");
			for(int x=0; x < oRows.Length; x++)
			{
				string orderID = oRows[x]["OrderID"]+"";
				DataRow[] eRows = execTable.Select("Order_ID = '" + orderID +"'");
				ArrayList rowArrayList = new ArrayList();
				for(int y=0; y < eRows.Length; y++)
				{
					if(eRows[y]["ReportedOn"] != System.DBNull.Value)
					{
						ArrayList rArrayList = GetStringArray(eRows[y]["ReportedOn"]+"");
						foreach(string r in rArrayList)
							if(!rowArrayList.Contains(r))
								rowArrayList.Add(r);
					}
				}
				string rowList = GetString(rowArrayList);
				oRows[x]["ReportedOn"] = rowList;
			}
		}


		protected string SetAR(string cFilter)
		{
			if(cFilter.Length > 0 && cFilter.IndexOf(" AND AllReported = 'false'") == -1)
				cFilter += " AND AllReported = 'false'";
			else
				cFilter = "AllReported = 'false'";

			return cFilter;
		}

		protected string RemAR(string cFilter)
		{
			if(cFilter.Length > 0)
			{
				int index = cFilter.IndexOf(" AND AllReported = 'false'");
				if(index != -1)
				{
					cFilter = cFilter.Remove(index,26);
					return cFilter;
				}
				else
				{
					index = cFilter.IndexOf("AllReported = 'false'");
					if(index != -1)
					{
						cFilter = cFilter.Remove(index,21);
						return cFilter;
					}
				}
			}

			return cFilter;
		}


		protected string SetUR(string cFilter)
		{
			if(cFilter.Length > 0)
			{
				if(cFilter.IndexOf("IsNull(ReportedOn,-1) = '-1'") == -1)
					cFilter +=  " AND IsNull(ReportedOn,-1) = '-1'";
			}
			else
			{
				cFilter = "IsNull(ReportedOn,-1) = '-1'";
			}

			return cFilter;
		}

		protected string RemUR(string cFilter)
		{
			if(cFilter.Length > 0)
			{
				int index = cFilter.IndexOf(" AND IsNull(ReportedOn,-1) = '-1'");
				if(index != -1)
				{
					cFilter = cFilter.Remove(index,33);
					return cFilter;
				}
				else
				{
					index = cFilter.IndexOf("IsNull(ReportedOn,-1) = '-1'");
					if(index != -1)
					{
						cFilter = cFilter.Remove(index,28);
						return cFilter;
					}
				}
			}

			return cFilter;
		}

		protected void ShowTab(bool bShow)
		{
			this.tabExecutions.SelectedTab = this.tabMatch;
			this.tabExecutions.Visible = (!bShow);
		}
		protected void SetErrorSafe(string text)
		{
			//if(labelStatus.ForeColor != Color.Red)
			Flash();
			statusBarPanel1.Text = text;
			toolTipExec.SetToolTip(statusBar1,text);
			//labelStatus.Text = text;
			//labelStatus.ForeColor = Color.Red;
		}
		protected void Flash()
		{
			try
			{
				flashCount = 0;
				this.Invoke(new MethodInvoker(this.test));
			}
			catch(Exception)
			{}
		}

		protected void test()
		{
			try
			{
				flashTimer.Enabled = true;
			}
			catch(Exception)
			{}
		}

		protected void FlashDialog(object sender, EventArgs e)
		{
			try
			{
				if(this.BackColor != Color.Red)
				{
					this.BackColor = Color.Red;
				}
				else 
				{
					this.BackColor = System.Windows.Forms.Form.DefaultBackColor;
					flashCount += 1;
					if(flashCount > flashCountMax )
					{
						flashTimer.Enabled = false;
						//						bFlashing = false;
					}
				}
			}
			catch(Exception)
			{}
		}


		protected void AddExecutionSafe2(Hashtable execution)
		{
			this.edbgMain.BeginUpdate();
			AddExecutionSafe(execution);
			DoExpansion();
			this.edbgMain.EndUpdate();
		}

		protected void AddExecutionsSafe2(IList executions)
		{
			this.edbgMain.BeginUpdate();
			AddExecutionsSafe(executions);
			DoExpansion();
			this.edbgMain.EndUpdate();
		}

		protected void AddExecutionSafe(Hashtable execution)
		{
			DataRow[] cRows = null;
			string repList = "";
			string newList = "";
			bool exists;

			//			this.edbgMain.BeginUpdate();
			
			if(orderTable.Rows.Contains(execution[Nodes.ORDERID]))
			{
				exists = UpdateExecution(execution, out repList, out newList, out cRows);
				UpdateOrders(execution[Nodes.ORDERID]+"",execution,exists, cRows, repList,newList);
			}
			else
			{
				AddOrder(execution[Nodes.ORDERID]+"");
				exists = UpdateExecution(execution, out repList, out newList, out cRows);
				UpdateOrders(execution[Nodes.ORDERID]+"",execution,false,null,repList,""); 
			}
         
			if(!exists)
				SetColorScheme(colorsHash);

			ResetMatchings();
			//			if(bUnreportedOnly)
			//				HideRows();
			//			this.edbgMain.EndUpdate();
		}
		protected void AddExecutionsSafe(IList executions)
		{
			if(executions == null)
				return;

			//			this.edbgMain.BeginUpdate();

			foreach(Hashtable exec in executions)
				AddExecutionSafe(exec);

			//			this.edbgMain.EndUpdate();
		}

		protected void DeleteExecutionSafe(Hashtable execution)
		{
			if(orderTable.Rows.Contains(execution[Nodes.ORDERID]))
			{
				string filter = "";

				IDictionaryEnumerator en = execution.GetEnumerator();
				while(en.MoveNext())
				{
					if(pkHash.ContainsKey(en.Key))
					{
						filter += pkHash[en.Key] + " = '" + en.Value + "' AND ";
					}
				}

				if(filter.Length>0)
					filter = filter.Remove(filter.Length - 5, 5);
            
				DataRow[] rows = execTable.Select(filter);

				if(rows.Length > 0)
				{
					foreach(DataRow row in rows)
						execTable.Rows.Remove(row);
					
					RemoveFromOrder(execution);
				}
					
				else
					return;
			}

			SetColorScheme(colorsHash);
			this.edbgMain.Refresh();

		}

		protected void DeleteExecutionsSafe(IList executions)
		{
			//			ArrayList list = executions as ArrayList;
			if(executions == null)
				return;

			foreach(Hashtable exec in executions)
			{
				//				Hashtable exec = obj as Hashtable;
				//				if(exec == null)
				//					continue;

				DeleteExecutionSafe(exec);
			}
		}

		protected void ClearExecutionsSafe()
		{
			this.edbgMain.BeginUpdate();
			execTable.Rows.Clear();
			orderTable.Rows.Clear();
			this.edbgMain.EndUpdate();
		}

		
		protected void RemoveFromOrder(Hashtable execution)
		{
			for(int y=0; y < orderTable.Rows.Count; y++)
			{
				string orderID = orderTable.Rows[y]["OrderID"]+"";
				if(orderID == execution[Nodes.ORDERID]+"")
				{
					DataRow[] rows = orderTable.Rows[y].GetChildRows("mainRelation");
					if(rows.Length > 0)
					{
						DataRow row = orderTable.Rows[y];

						if((row["Quantity"]+"") != "" && (execution[Nodes.QUANTITY]+"") != "")
						{
							//							long qty1=0;
							//							ValidateData.Interpret.Quantity(row["Quantity"]+"", out qty1);
							//							long qty2=0;
							//							ValidateData.Interpret.Quantity(execution[Nodes.QUANTITY]+"", out qty2);
							row["Quantity"] = (long.Parse(row["Quantity"]+"")) - long.Parse((execution[Nodes.QUANTITY]+""))+""; /*(qty1-qty2).ToString();*/
							row["Price"] = CalcPrice(orderID)/(long.Parse(row["Quantity"]+""));  /*(qty1-qty2);*/
						}

						//						
						//						long lQty = long.Parse(row["Quantity"]+"");
						//						double dPrice = double.Parse(row["Price"]+"");
						//						double dTotal = lQty * dPrice;
						//
						//						int items = (int)Math.Ceiling(lQty/dPrice);
						//						long nQty = lQty - long.Parse(execution[Nodes.QUANTITY]+"");
						//						row["Quantity"] = nQty;

						row["ReportedOn"] = GetNew(row["ReportedOn"], execution[Nodes.ReportedOn]);
					}

					else
					{
						orderTable.Rows.RemoveAt(y);
						y++;
					}
				}				
			}
		}

		protected string GetNew(object obj, object exec)
		{
			string str = obj as string;
			if(str == null)
				return "";

			IList list = exec as IList;

			string str2 = "";

			if(list != null)
			{
				
				foreach(string s in list)
					str2 += s + "," ;
			}

			if(str2.Length > 0)
				str2 = str2.Remove(str2.Length-1,1);

			int index = str.IndexOf(str2);
			if(index > 0)
				str = str.Remove(index-1,str2.Length);
			else if(index == 0)
				str = str.Remove(index,str2.Length);
            
			return str;


		}

		protected bool IsOnTop()
		{
			return (this.tabExecutions.SelectedTab == this.tabMatch);
		}

		protected void SetTop()
		{
			this.tabExecutions.SelectedTab = this.tabMatch;
		}			

		protected void SetBottom()
		{
			this.tabExecutions.SelectedTab = this.tabUnmatch;
		}
		
		protected void AddOrder(string orderID)
		{
			DataRow dr = orderTable.NewRow();
			dr["OrderID"] = orderID;
			dr["Quantity"] = 0;
			dr["Price"] = 0;
			dr["Product"] = "";
			dr["Symbol"] = "";
			dr["Market"] = "";
			dr["ReportedOn"] = System.DBNull.Value;
			dr["ShowAsMatched"] = "";
			dr["RefNum"] = "";
			dr["AllReported"] = "false";
			dr["Counter"] = 0;
			orderTable.Rows.Add(dr);
			if(!expansionHash.Contains(orderID))
				expansionHash[orderID] = true;
		}


		protected bool UpdateExecution(Hashtable execution, out string reported, out string newList, out DataRow[] rows)
		{
			bool exists = false;
			rows = null;
			reported = "";
			newList = "";
				
			string filter = "";


			IDictionaryEnumerator en = execution.GetEnumerator();
			while(en.MoveNext())
			{
				if(pkHash.ContainsKey(en.Key))
				{
					filter += pkHash[en.Key] + " = '" + en.Value + "' AND ";
				}
			}

			if(filter.Length>0)
				filter = filter.Remove(filter.Length - 5, 5);
            
			rows = execTable.Select(filter);

			if(rows.Length > 0)
				exists = true;
			
			if(!exists)
			{
				
				DataRow dr = execTable.NewRow();
				dr["Key"] = ++key;
				dr["Action"] = GetJargons(execution[Nodes.Action]+"");
				dr["Quantity"] = execution[Nodes.QUANTITY];
				dr["Rem. Qty"] = execution[Nodes.REMQUANTITY];
				dr["Symbol"] = execution[Nodes.SYMBOL];
				dr["Price"] = execution[Nodes.PRICE];
				dr["Market"] = execution[Nodes.Market];
				dr["ReportedOn"] = System.DBNull.Value;
				dr["RefNum"] = execution[Nodes.REFNUM];
				dr["SeqNum"] = execution[Nodes.SEQNUM];
				dr["TimeStamp"] = execution[Nodes.TIMESTAMP];
				dr["Order_ID"] = execution[Nodes.ORDERID];
				dr["ShowAsMatched"] = execution[Nodes.ShowAsMatched];
				dr["BrokerID"] = execution[Nodes.Broker];
				dr["Product"] = execution[Nodes.PRODUCT];
				dr["ExtraInfo"] = "";
				dr["OIRN"] = dr["Order_ID"] + "/" + dr["RefNum"];
					
				if(execution[Nodes.ReportedOn] != null || execution[Nodes.ReportedOn]+"" != string.Empty)
				{
					IList tempList = execution[Nodes.ReportedOn] as IList;

					if(tempList != null && tempList.Count > 0)
					{
						foreach(string rep in tempList)
						{
							reported += rep + ",";
						}
						if(reported.Length>0)						
							reported = reported.Remove(reported.Length-1,1);
					}
				}
				if(reported == null || reported.Length==0)
					dr["ReportedOn"] = System.DBNull.Value;
				else
					dr["ReportedOn"] = reported;

				execTable.Rows.Add(dr);
				
			}
			else
			{
				//					this.edbgMain.Binder.SuspendBinding();
				//foreach(DataRow row in rows)
				for(int x=0; x < rows.Length; x++)
				{
					DataRow row = rows[x];
					if(execution[Nodes.QUANTITY] != null)
						row["Quantity"] = execution[Nodes.QUANTITY];
					if(execution[Nodes.PRICE] != null)
						row["Price"] = execution[Nodes.PRICE];
					if(execution[Nodes.REFNUM] != null)
						row["RefNum"] = execution[Nodes.REFNUM];
					if(execution[Nodes.TIMESTAMP] != null)
						row["TimeStamp"] = execution[Nodes.TIMESTAMP];
					if(execution[Nodes.Broker] != null)
						row["BrokerID"] = execution[Nodes.Broker];
					if(execution[Nodes.PRODUCT] != null)
						row["Product"] = execution[Nodes.PRODUCT];
					if(execution[Nodes.ShowAsMatched] != null)
						row["ShowAsMatched"] = execution[Nodes.ShowAsMatched];
					if(execution[Nodes.REMQUANTITY] != null)
						row["Rem. Qty"] = execution[Nodes.REMQUANTITY];
                    
					//					DateTime dt1 = System.DateTime.Now;
					string str = "";
					if(execution[Nodes.ReportedOn] != null)
					{
						IList eList = execution[Nodes.ReportedOn] as IList;
						if(eList != null && eList.Count > 0)
						{
							
							foreach(string e in eList)
								str += e + ",";
							if(str.Length>0)
								str = str.Remove(str.Length-1,1);

							//row["ReportedOn"] = str;
						}
					}
					if(str == null || str.Length==0)
					{
						reported = row["ReportedOn"]+"";
						row["ReportedOn"] = System.DBNull.Value;
						//						newList = "";
					}
					else
					{
						reported = row["ReportedOn"]+"";
						newList = str;
						row["ReportedOn"] = str;						
					}
					//					DateTime dt21 = System.DateTime.Now;
					//
					//					TimeSpan t1 = (TimeSpan)(dt21-dt1);
					//					System.Diagnostics.Debug.WriteLine("UE --> " + t1.TotalMilliseconds);
					row["OIRN"] = row["Order_ID"] + "/" + row["RefNum"];

				}
		
				
			}
			
            
			return exists;
		}

		/*
		protected void UpdateValues()
		{
			this.edbgMain.BeginUpdate();
			GridCurrentCell cc = this.edbgMain.CurrentCell;
			GridBoundRecordState rs = this.gridBinder.GetRecordStateAtRowIndex(cc.RowIndex);
			
			if(cc != null & rs != null & rs.Table != null & rs.Position >= 0 && rs.Position <= rs.Table.Count && rs.LevelIndex == 0 )
			{
				this.edbgMain.BeginUpdate();
				cc.EndEdit();
				this.gridBinder.EndEdit();
				if(this.edbgMain.IsExpandedAtRowIndex(cc.RowIndex))
				{
					this.edbgMain.CollapseAtRowIndex(cc.RowIndex);
					this.edbgMain.ExpandAtRowIndex(cc.RowIndex);
				}
						
				cc.MoveTo(cc.RowIndex, cc.ColIndex, GridSetCurrentCellOptions.SetFocus);
				this.edbgMain.EndUpdate();
			}
		}

		protected int GetRowIndex(DataRow dr)
		{
			DataView cView = this.edbgMain.DataSource as DataView;

			if(cView == null)
				return 0;

			int counter = 0;
			IEnumerator en = cView.GetEnumerator();
			while(en.MoveNext())
			{
				counter++;
				DataRowView drv = en.Current as DataRowView;
				if(drv == null)
					continue;

				if ( drv.Row == dr)
					return counter;
			}

			return 0;
		}
*/

		
		protected void UpdateOrders(string orderID, Hashtable execution, bool bExists, DataRow[] rows, string rList, string oList)
		{
			if(!bExists)
			{
				foreach(DataRow row in orderTable.Rows)
				{
					if ((row["OrderID"]+"") == orderID)
					{
						if((row["Quantity"]+"") != "" && (execution[Nodes.QUANTITY]+"") != "")
						{
							//							long qty1 = 0;
							//							long qty2 = 0;
							//							ValidateData.Interpret.Quantity(row["Quantity"]+"", out qty1);
							//							ValidateData.Interpret.Quantity(execution[Nodes.QUANTITY]+"", out qty2);

							row["Quantity"] = (long.Parse(row["Quantity"]+"")) + long.Parse((execution[Nodes.QUANTITY]+""))+""; //(qty1-qty2).ToString();
							row["Price"] = CalcPrice(orderID)/(long.Parse(row["Quantity"]+"")); //(qty1-qty2);
						}
						row["ShowAsMatched"] = execution[Nodes.ShowAsMatched]+"";
						row["Action"] = GetJargons(execution[Nodes.Action]+"");
						row["Symbol"] = execution[Nodes.SYMBOL]+"";
						row["Market"] = execution[Nodes.Market]+"";
						row["Product"] = execution[Nodes.PRODUCT]+"";

						if(rList.Length > 0 && execution[Nodes.ReportedOn] != null)
						{
							string rowList = row["ReportedOn"]+"";
							string str = rowList;

							ArrayList rAList = GetStringArray(rList);
							foreach(string r in rAList)
								if(rowList.IndexOf(r) == -1)
									str += "," + r ;

							if(rAList.Count == 0)
								str = rowList;
							if(str.StartsWith(","))
								str = str.Remove(0,1);
							if(str.EndsWith(","))
								str = str.Remove(str.Length-1,1);
							row["ReportedOn"] = str;
							//
							//							long val=0;
							//							ValidateData.Interpret.Quantity(row["Counter"]+"", out val);

							row["Counter"] = long.Parse(row["Counter"]+"") + 1; // val;

							if( /*val*/long.Parse(row["Counter"]+"") == GetOrderRows(orderID).Count)
								row["AllReported"] = "true";
							else
								row["AllReported"] = "false";
						}
						else
							row["AllReported"] = "false";
						//						{
						//							IList myList = execution[Nodes.ReportedOn] as IList;
						//						if(myList.Count == 0)
						//								row["ReportedOn"] = System.DBNull.Value;
						//
						//					}                            							
						
						row["RefNum"] = execution[Nodes.REFNUM]+"";
						row["OIRN"] = row["OrderID"] + "/" + row["RefNum"];
					}
				}
			}
			else
			{
				if(rows != null)
				{
					DataRow[] oRows = orderTable.Select("OrderID = '" + orderID +"'");
					foreach(DataRow row in oRows)
					{
						//						if ((row["OrderID"]+"") == orderID)
						//						{
						foreach(DataRow dr in rows)
						{
							if(dr["Quantity"] != null && dr["Quantity"]+"" != "")
							{
								long qty = long.Parse(dr["Quantity"]+"");
								//									ValidateData.Interpret.Quantity(dr["Quantity"]+"", out qty);
									
								double price = double.Parse(dr["Price"]+"");
								double tominus = qty * price;

								//long val =0;
								//ValidateData.Interpret.Quantity(execution[Nodes.QUANTITY]+"", out val);
								if(execution[Nodes.QUANTITY] != null  && qty != long.Parse(execution[Nodes.QUANTITY]+"") /*val*/)
								{	
									//										long tq = 0;
									//										ValidateData.Interpret.Quantity(row["Quantity"]+"", out tq);

									row["Quantity"] = /*tq*/long.Parse(row["Quantity"]+"") - qty + /*val*/long.Parse((execution[Nodes.QUANTITY]+""));
								}
							
								if(execution[Nodes.PRICE] != null && price != double.Parse(execution[Nodes.PRICE]+"") )
								{
									//										long tq = 0;
									//										ValidateData.Interpret.Quantity(row["Quantity"]+"", out tq);

									double cPrice = double.Parse(row["Price"]+"") * long.Parse(row["Quantity"]+"")/*tq*/;
									cPrice = cPrice - tominus;
									cPrice = cPrice + ( double.Parse(execution[Nodes.PRICE]+"") * /*val*/long.Parse(execution[Nodes.QUANTITY] +""));
									row["Price"] = cPrice;
								}
							}

							row["ShowAsMatched"] = execution[Nodes.ShowAsMatched];

							#region ReCalc

							DataRow[] eRows = execTable.Select("Order_ID = '" + orderID +"'");
							ArrayList rowArrayList = new ArrayList();
							for(int y=0; y < eRows.Length; y++)
							{
								if(eRows[y]["ReportedOn"] != System.DBNull.Value)
								{
									ArrayList rArrayList = GetStringArray(eRows[y]["ReportedOn"]+"");
									foreach(string r in rArrayList)
										if(!rowArrayList.Contains(r))
											rowArrayList.Add(r);
								}
							}
							string rowList = GetString(rowArrayList);
							row["ReportedOn"] = rowList;

							ArrayList _oArrayList = GetStringArray(oList);
							ArrayList _rArrayList = GetStringArray(rList);

							if(_oArrayList.Count > 0)
							{
								if(_rArrayList.Count < 1)
								{
									row["Counter"] = int.Parse(row["Counter"]+"") + 1;

									if( int.Parse(row["Counter"]+"") == GetOrderRows(orderID).Count)
										row["AllReported"] = "true";
								}
							}
							else
							{
								if(_oArrayList.Count != _rArrayList.Count)
								{
									row["Counter"] = int.Parse(row["Counter"]+"") - 1;
								}
								row["AllReported"] = "false";
							}
						
							#endregion

							//							string rowList = row["ReportedOn"]+"";

							#region Latest Commented
							/*
															ArrayList rowArrayList = GetStringArray(rowList);
															ArrayList oArrayList = GetStringArray(oList);
															ArrayList rArrayList = GetStringArray(rList);

															bool doSomething = false;
															foreach(string r in rArrayList)
															{
																string _filter = "Order_ID = '" + orderID + "' AND ReportedOn LIKE '*" + r + "*'";
																DataRow[] fRows = execTable.Select(_filter);

																if(fRows.Length > 0)
																	doSomething = false;
																else
																	doSomething = true;
															}

															if(oArrayList.Count > 0)
															{
																if(doSomething)
																{
																	foreach(string s in rArrayList)
																	{
																		if(rowArrayList.Contains(s))
																			rowArrayList.Remove(s);
																	}
																}

																foreach(string s in oArrayList)
																{
																	if(!rowArrayList.Contains(s))
																		rowArrayList.Add(s);
																}								

																if(rArrayList.Count < 1)
																{
																	row["Counter"] = int.Parse(row["Counter"]+"") + 1;

																	if( int.Parse(row["Counter"]+"") == GetOrderRows(orderID).Count)
																		row["AllReported"] = "true";
																}
															}

															else
															{
																if(oArrayList.Count != rArrayList.Count)
																{
																	row["Counter"] = int.Parse(row["Counter"]+"") - 1;
							//										row["AllReported"] = "false";
																}

																if(doSomething)
																{
																	foreach(string s in rArrayList)
																	{
																		if(rowArrayList.Contains(s))
																			rowArrayList.Remove(s);
																	}

																	foreach(string s in oArrayList)
																	{
																		if(!rowArrayList.Contains(s))
																			rowArrayList.Add(s);
																	}                                        										
																}
																row["AllReported"] = "false";
															}
							


															row["ReportedOn"] = GetString(rowArrayList);
							  */
								#endregion
                              
						}
						//						}
					}
				}
			}
		}


		protected string GetString(ArrayList sList)
		{
			string str = "";

			foreach(string s in sList)
				str += s + "," ;
		
			if(str.StartsWith(","))
				str = str.Remove(0,1);

			if(str.EndsWith(","))
				str = str.Remove(str.Length-1,1);

			if(str.IndexOf(",,") != -1)
				str = str.Remove(str.IndexOf(",,"),1);

			return str;
		}

		protected ArrayList GetStringArray(string str)
		{
			ArrayList sList = new ArrayList();

			string[] strList = System.Text.RegularExpressions.Regex.Split(str,",");
			foreach(string s in strList)
				sList.Add(s);

			if(sList.Count == 1)
				if(sList[0].ToString() == "")
					sList.Clear();

			return sList;
		}


		protected ArrayList GetOrderRows(string orderID)
		{
			ArrayList rowList = new ArrayList();
			foreach(DataRow row in execTable.Rows)
			{
				if(row["Order_ID"]+"" == orderID)
					rowList.Add(row);
			}
			return rowList;
		}

		protected double CalcPrice(string oID)
		{
			double total = 0;

			foreach(DataRow row in execTable.Rows)
				if((row["Order_ID"]+"") == oID)
				{
					//				long qty=0;
					//				ValidateData.Interpret.Quantity(row["Quantity"]+"", out qty);
					total += (double.Parse(row["Price"]+"") * long.Parse(row["Quantity"]+"")/*qty*/);
				}
			
			return total;
		}
         
		/// <summary>
		/// Setup DataTables, DataViews and DataSet
		/// </summary>
		protected void Setup()
		{

			this.gridBinder = this.edbgMain.Binder;

			setExecutions = new DataSet("setExecutions");
			execTable = new DataTable("execTable");
			orderTable = new DataTable("orderTable");

			execTable.Columns.Add("Key");
			execTable.Columns.Add("Action");
			execTable.Columns.Add("Quantity",typeof(long));
			execTable.Columns.Add("Rem. Qty",typeof(long));
			execTable.Columns.Add("Symbol");
			execTable.Columns.Add("Price",typeof(double));
			execTable.Columns.Add("Product");
			execTable.Columns.Add("Market");
			execTable.Columns.Add("ReportedOn");
			execTable.Columns.Add("RefNum");
			execTable.Columns.Add("SeqNum");
			execTable.Columns.Add("TimeStamp");
			execTable.Columns.Add("Order_ID");
			execTable.Columns.Add("ShowAsMatched");
			execTable.Columns.Add("BrokerID");
			execTable.Columns.Add("ExtraInfo");
			execTable.Columns.Add("OIRN");
			execTable.PrimaryKey = new DataColumn[] { execTable.Columns["Key"]};//, execTable.Columns["Order_ID"] };


			orderTable.Columns.Add("OrderID");
			orderTable.Columns.Add("Action");
			orderTable.Columns.Add("Quantity",typeof(long)); // TotalQuantity
			orderTable.Columns.Add("Symbol");
			orderTable.Columns.Add("Price",typeof(double)); // (Sum of Pi & Qi) div (Sum Qi)
			orderTable.Columns.Add("Product");
			orderTable.Columns.Add("Market");
			orderTable.Columns.Add("ReportedOn"); // concat list of rep on
			orderTable.Columns.Add("RefNum");
			orderTable.Columns.Add("ShowAsMatched");
			orderTable.Columns.Add("Counter");
			orderTable.Columns.Add("AllReported");
			orderTable.Columns.Add("OIRN");

			orderTable.PrimaryKey = new DataColumn[] { orderTable.Columns["OrderID"] };

			setExecutions.Tables.Add(orderTable);
			setExecutions.Tables.Add(execTable);

			setExecutions.Relations.Add("mainRelation",orderTable.Columns["OrderID"],execTable.Columns["Order_ID"],false);

			orderView = new DataView(setExecutions.Tables["orderTable"]);
			execView = new DataView(setExecutions.Tables["execTable"]);

			orderView.Sort = "OrderID ASC";
			execView.Sort = "SeqNum ASC";

			orderTable.DefaultView.Sort = "OrderID ASC";
			execTable.DefaultView.Sort = "SeqNum ASC";

			this.orderTable.DefaultView.ListChanged += new ListChangedEventHandler(this.OnListChange);

			//			hlr = gridBinder.AddRelation("mainRelation");
		}


		protected string GetJargons(string oStr)
		{
			string nStr = oStr;
			if(jargonsHash != null && jargonsHash.ContainsKey(oStr))
				nStr = jargonsHash[oStr]+"";

			return nStr;
		}

		
		protected string GetFilter(string cFilter, bool bMatchedFilter)
		{
			if(cFilter != string.Empty)
			{
				if(cFilter.IndexOf("ShowAsMatched = '") == -1)
					return cFilter + " AND ShowAsMatched = '" + bMatchedFilter + "'";
				else
				{
					//					if(bMatchedFilter)
					//					{
					cFilter = cFilter.Replace("ShowAsMatched = '" + (!bMatchedFilter) + "'","ShowAsMatched = '" + bMatchedFilter + "'");
					return cFilter;
					//					}
					//					else
					//					{
					//						cFilter = cFilter.Replace("ShowAsMatched = 'true'","ShowAsMatched = 'false'");
					//						return cFilter;
					//					}
				}
			}

			return "ShowAsMatched = '" + bMatchedFilter + "'";
		}

		/*
		protected string GetMatchedFilter(string cFilter)
		{
			if(cFilter != string.Empty)
			{
				if(cFilter.IndexOf("ShowAsMatched = '") == -1)
					return cFilter + " AND ShowAsMatched = 'True'";
				else
				{
					cFilter = cFilter.Replace("ShowAsMatched = 'False'","ShowAsMatched = 'True'");
					return cFilter;
				}
			}			

			return "ShowAsMatched = 'True'";
		}

		protected string GetUnmatchedFilter(string cFilter)
		{

			if(cFilter != string.Empty)
			{
				if(cFilter.IndexOf("ShowAsMatched = '") == -1)
					return cFilter + " AND ShowAsMatched = 'False'";
				else
				{
					cFilter = cFilter.Replace("ShowAsMatched = 'True'","ShowAsMatched = 'False'");
					return cFilter;
				}
			}			

			return "ShowAsMatched = 'False'";
		}

*/

		protected Hashtable GetExecutionHash(DataRow row)
		{
			Hashtable execHash = new Hashtable();
			execHash[Nodes.ORDERID] = row["Order_ID"];
			execHash[Nodes.SEQNUM] = row["SeqNum"];
			execHash[Nodes.Action] = row["Action"];
			execHash[Nodes.QUANTITY] = row["Quantity"];
			execHash[Nodes.REMQUANTITY] = row["Rem. Qty"];
			execHash[Nodes.SYMBOL] = row["Symbol"];
			execHash[Nodes.PRICE] = row["Price"];
			execHash[Nodes.Market] = row["Market"];
			execHash[Nodes.REFNUM] = row["RefNum"];
			execHash[Nodes.ShowAsMatched] = row["ShowAsMatched"];
			execHash[Nodes.TIMESTAMP] = row["TimeStamp"];
			execHash[Nodes.Broker] = row["BrokerID"];
			execHash[Nodes.PRODUCT] = row["Product"];
			//			execHash[Nodes.ExtraInfo] = row["ExtraInfo"};
			string str = row["ReportedOn"] as string;
			if(str != null)
			{
				IList aList = new ArrayList();
				string[] strs = System.Text.RegularExpressions.Regex.Split(str,",");
				foreach(string _str in strs)
					aList.Add(_str);
				execHash[Nodes.ReportedOn] = aList;
			}
			return execHash;
		}

		protected IList GetExecutions()
		{
			ArrayList execList = new ArrayList();
			foreach(DataRow row in execTable.Rows)
			{
				execList.Add(GetExecutionHash(row));
			}
			return execList;
		}

		protected IList GetMatchedExecutions()
		{
			ArrayList mList = new ArrayList();
			foreach(DataRow row in execTable.Rows)
			{
				if(row["ShowAsMatched"]+"" == "true")
					mList.Add(GetExecutionHash(row));
			}
			return mList;
		}

		protected IList GetUnmatchedExecutions()
		{
			ArrayList uList = new ArrayList();
			foreach(DataRow row in execTable.Rows)
			{
				if(row["ShowAsMatched"]+"" == "false")
					uList.Add(GetExecutionHash(row));
			}
			return uList;
		}

		protected IList GetSelectedOrders()
		{
			ArrayList sList = new ArrayList();

			GridModelSelections gms = this.edbgMain.Selections;

			if(gms == null)
				return sList;
			
			foreach(GridRangeInfo gri in gms.Ranges)
			{
				if(gri == null)
					continue;
				
				for(int x = gri.Top; x <= gri.Bottom; x++)
				{
					GridBoundRecordState gbrs = edbgMain.Binder.GetRecordStateAtRowIndex(x);
					if(gbrs != null && gbrs.Table != null & gbrs.Position >= 0 && gbrs.Position < gbrs.Table.Count)
					{
						if(gbrs.Table[gbrs.Position] is DataRowView)
						{

							DataRow row = (gbrs.Table[gbrs.Position] as DataRowView).Row;
							if(row.Table.Columns.Contains("OrderID"))
							{
								// sList.Add(GetOrderHash());
								Hashtable ht = new Hashtable();
								ht[Nodes.ISORDER] = true;
								sList.Add(ht);
							}
						}
					}
				}
			}

			return sList;
		}


		protected IList GetSelectedExecutions()
		{
			ArrayList sList = new ArrayList();

			GridModelSelections gms = this.edbgMain.Selections;

			if(gms == null)
				return sList;

			foreach(GridRangeInfo gri in gms.Ranges)
			{
				if(gri == null)
					continue;
				
				for(int x = gri.Top; x <= gri.Bottom; x++)
				{
					GridBoundRecordState gbrs = edbgMain.Binder.GetRecordStateAtRowIndex(x);
					if(gbrs != null && gbrs.Table != null & gbrs.Position >= 0 && gbrs.Position < gbrs.Table.Count)
					{
						if(gbrs.Table[gbrs.Position] is DataRowView)
						{

							DataRow row = (gbrs.Table[gbrs.Position] as DataRowView).Row;
							if(row.Table.Columns.Contains("Order_ID"))
								sList.Add(GetExecutionHash(row));
							else
							{
								ArrayList orList = GetOrderRows(row["OrderID"]+"");
								foreach(DataRow orderRow in orList)
									sList.Add(GetExecutionHash(orderRow));

								//								Hashtable ht = new Hashtable();
								//								ht[Nodes.ISORDER] = true;
								//								sList.Add(ht);
							}
						}
					}
				}
			}

			return sList;
		}


		protected void GroupSafe()
		{
			this.SuspendLayout();
			this.edbgMain.BeginUpdate();
			Group();
			DoExpansion();
			this.edbgMain.EndUpdate();
			this.ResumeLayout(false);
		}

		protected void Group()
		{

			//			this.SuspendLayout();
			
			//			this.edbgMain.BeginUpdate();

			if(!bGroup)
			{
			
				this.edbgMain.DataSource = null;
				this.edbgMain.DataSource = orderTable.DefaultView; //orderView;
				
				try
				{
					hlr = gridBinder.AddRelation("mainRelation");

					if(hlr != null)
					{

						this.gridBinder.GetHierarchyLevel(0).LayoutColumns(new string[] {
																							//			"OrderID",
																							"Action",
																							"Quantity",
																							"Symbol",
																							"Price",
																							"Market",
																							"ReportedOn",
																							"OIRN"
																						});

						hlr.LayoutColumns( new string[] {
															//		"OrderID",
															"Action",
															"Quantity",
															"Symbol",
															"Price",
															"Market",
															"ReportedOn",
															"OIRN"
														});

						foreach(GridBoundColumn gbc in hlr.InternalColumns)
						{
							gbc.StyleInfo.CellType = "Static";
							gbc.StyleInfo.ReadOnly = true;
							gbc.StyleInfo.Trimming = StringTrimming.EllipsisCharacter;
							if(gbc.MappingName != "Action")
								gbc.StyleInfo.HorizontalAlignment = GridHorizontalAlignment.Center;
							if(gbc.MappingName == "Quantity")
							{
								gbc.StyleInfo.CellValueType = typeof(System.UInt64);
								gbc.StyleInfo.Format = "#,###,###";
							}
							if(gbc.MappingName == "Price")
							{
								gbc.StyleInfo.CellValueType = typeof(System.Double);
								gbc.StyleInfo.Format = "0.0000";
							}
						}
						
						hlr.ShowHeaders = false;
						
						
					}
					
				}
				catch(Exception)
				{}

				ResetMatchings();
				SetUnreported(bUnreportedOnly);
				//		DoExpansion();
				//this.edbgMain.ExpandAll();
				//				this.edbgMain.EndUpdate();

				//				this.ResumeLayout(false);
			}


		}
		/*
		protected void HideUnwanted(bool bHide)
		{

			ArrayList intList = new ArrayList();

			for(int x=0; x < edbgMain.Model.RowCount; x++)
			{
				GridBoundRecordState gbrs = this.gridBinder.GetRecordStateAtRowIndex(x);

				if(gbrs != null && gbrs.Table != null && gbrs.Position >= 0 && gbrs.Position < gbrs.Table.Count)
				{
					if(gbrs.LevelIndex == 1)
					{
						DataRowView drv = gbrs.Table[gbrs.Position] as DataRowView;
						if (drv.Row["ReportedOn"] == System.DBNull.Value)
							intList.Add(x);
					}
				}
			}

	

			foreach(int i in intList)
			{
				edbgMain.Model.Rows.Hidden[i] = bHide;
			}
		}
*/	


		protected void UnGroupSafe()
		{
			this.SuspendLayout();
			this.edbgMain.BeginUpdate();
			UnGroup();
			this.edbgMain.EndUpdate();
			this.ResumeLayout(false);
		}
		protected void UnGroup()
		{

			//			this.edbgMain.SuspendLayout();
			//			this.edbgMain.BeginUpdate();
			
			if(bGroup)
			{
				this.edbgMain.DataSource = null;
				this.edbgMain.DataSource = execTable.DefaultView /*execView*/;
			}
			
			ResetMatchings();
			SetUnreported(bUnreportedOnly);

			//			this.edbgMain.EndUpdate();

			//			this.edbgMain.ResumeLayout(false);
		}

		protected void ResetMatchings()
		{
			DataView currentView = this.edbgMain.DataSource as DataView;
			if(currentView == null)
				return;

			if(this.tabExecutions.SelectedTab == this.tabMatch)
				currentView.RowFilter = GetFilter(currentView.RowFilter,true);
			else
				currentView.RowFilter = GetFilter(currentView.RowFilter,false);


			//			this.edbgMain.Refresh();
			
			//			if(currentView == orderTable.DefaultView /*orderView*/)
			//				DoExpansion();

			
		}			
	
		protected void OnlyMatched()
		{
			if(this.tabExecutions.Controls.Contains(this.tabUnmatch))
				this.tabExecutions.Controls.Remove(this.tabUnmatch);
		}

		protected void Both()
		{
			if(!this.tabExecutions.Controls.Contains(this.tabUnmatch))
				this.tabExecutions.Controls.Add(this.tabUnmatch);
		}
		
		protected void SetStatus(string text)
		{
			flashTimer.Stop();
			this.BackColor = System.Windows.Forms.Form.DefaultBackColor;
			this.statusBarPanel1.Text = text;
			toolTipExec.SetToolTip(statusBar1,text);
			//labelStatus.ForeColor = Color.Black;
			//labelStatus.Text = text;
		}

		protected void SetSettings(Hashtable settings)
		{
			if(settings == null)
				return;

			if(settings[Nodes.Rectangle] is Rectangle)
			{
				Rectangle r = (Rectangle)settings[Nodes.Rectangle];
				this.Height = r.Height;
				this.Top = r.Top;
				this.Left = r.Left;
				this.Width = r.Width;
				this.Location = r.Location;

			}
			if(settings[Nodes.Visible] != null)
				this.Visible = bool.Parse(settings[Nodes.Visible]+"");
			if(settings[Nodes.GroupExecution] != null)
				this.checkBoxGroup.Checked = bool.Parse(settings[Nodes.GroupExecution]+"");
			if(settings[Nodes.UnReportedOnly] != null)
				this.checkBoxUnreportedOnly.Checked = bool.Parse(settings[Nodes.UnReportedOnly]+"");
			if(settings[Nodes.ColWidths] != null)
			{
				IList list = settings[Nodes.ColWidths] as IList;
				if(list != null)
				{
					int counter = 0;
					foreach(int width in list)
					{
						counter++;
						this.edbgMain.SetColWidth(counter,counter,width);
					}
				}
			}
		}

		protected void SetColorScheme(Hashtable scheme)
		{
			this.edbgMain.RowStyle.Clear();
			if(scheme==null)return;
			foreach(DictionaryEntry de in scheme)
				SetRowStyle(de.Key as string, de.Value as Hashtable);

			//			Hashtable h = scheme[Nodes.Selection] as Hashtable;
			//			if(h != null)
			//				if(h[Nodes.BackColor] is Color)
			//					this.edbgMain.AlphaBlendSelectionColor = Color.FromArgb(255,(Color)h[Nodes.BackColor]);

			this.edbgMain.Refresh();
		}

		
		protected void SetRowStyle(string action,Hashtable style)
		{
			if(action == null || style ==  null)return;
			DataRow[] drs = execTable.Select("Action = '"+action+"'");
			foreach(DataRow dr in drs)
			{
				string key = dr["Key"]+"";//+ "," + dr["Order_ID"];
				string oid = dr["Order_ID"]+"";
				if(style[Nodes.ForeColor]is Color)
				{
					edbgMain.RowStyle[key].TextColor = (Color)style[Nodes.ForeColor];
					edbgMain.RowStyle[oid].TextColor = (Color)style[Nodes.ForeColor];
				}

				if(style[Nodes.BackColor]is Color)
				{
					edbgMain.RowStyle[key].BackColor = (Color)style[Nodes.BackColor];
					edbgMain.RowStyle[oid].BackColor = (Color)style[Nodes.BackColor];
				}
				if(style[Nodes.Font]is System.Drawing.Font)
				{
					edbgMain.RowStyle[key].Font = ToSynfusionFont(style[Nodes.Font] as System.Drawing.Font);
					edbgMain.RowStyle[oid].Font = ToSynfusionFont(style[Nodes.Font] as System.Drawing.Font);
				}

				//edbgMain.RowStyle[oid].Font.Bold = true;
				
			}
		}
		protected Syncfusion.Windows.Forms.Grid.GridFontInfo ToSynfusionFont(System.Drawing.Font font)
		{
			Syncfusion.Windows.Forms.Grid.GridFontInfo gfi = new Syncfusion.Windows.Forms.Grid.GridFontInfo();
			gfi.Facename = font.FontFamily.Name;
			gfi.Bold = font.Bold;
			gfi.FontStyle = font.Style;
			gfi.Italic = font.Italic;
			gfi.Size = font.Size;
			gfi.Strikeout = font.Strikeout;
			gfi.Underline = font.Underline;
			return gfi;
		}


		protected void DoExpansion()
		{
						
			for(int x=0; x < this.edbgMain.Model.RowCount; x++)
			{
				GridBoundRecordState gbrs = this.gridBinder.GetRecordStateAtRowIndex(x+1);
				if(gbrs != null && gbrs.Table != null && gbrs.Position >=0 && gbrs.Position < gbrs.Table.Count)
				{
					if(gbrs.LevelIndex == 0)
					{
						DataRowView drv = gbrs.Table[gbrs.Position] as DataRowView;

						if(drv != null && ((DataView)this.edbgMain.DataSource == orderTable.DefaultView )) //orderView
						{
							bool val = bool.Parse(expansionHash[drv.Row["OrderID"]+""]+"");

							if(val && (!this.edbgMain.IsExpandedAtRowIndex(x+1)))
							{
								bCheckExpansion = false;
								this.edbgMain.CollapseAtRowIndex(x+1);
								bCheckExpansion = true;
								this.edbgMain.ExpandAtRowIndex(x+1);
							}
							//							else
							//								this.edbgMain.CollapseAtRowIndex(x+1);
							
						}
					}
				}
			}
		}
		
		#endregion

		#region Event Handlers

		private void OnFormVisibleChanged(object sender, System.EventArgs e)
		{
			if(Visible)
			{
				this.SuspendLayout();
				if(this.tabExecutions.SelectedTab == this.tabMatch)
				{
					this.tabExecutions.SelectedTab = this.tabUnmatch;
					this.tabExecutions.SelectedTab = this.tabMatch;
					
				}
				else
				{
					this.tabExecutions.SelectedTab = this.tabMatch;
					this.tabExecutions.SelectedTab = this.tabUnmatch;
				}
				this.ResumeLayout(false);
			}
		}

		private void OnDoubleClick(Hashtable rowHash, GridCellClickEventArgs e)
		{
			if (OnRowDoubleClick != null)
				OnRowDoubleClick(this,e);
		}

		private void OnReConnectClick(object sender, System.EventArgs e)
		{
			if(OnReconnectClick != null)
				OnReconnectClick(this,e);
		}
		private void onConfigureClick(object sender, System.EventArgs e)
		{
			if(OnConfigure != null)
				OnConfigure(this,e);		
		}

		private void OnGridCellClick(object sender, Syncfusion.Windows.Forms.Grid.GridCellClickEventArgs e)
		{
			if(e.RowIndex == 0)
			{
				this.edbgMain.BeginUpdate();
				bCheckExpansion = false;
				this.edbgMain.CollapseAll();
				bCheckExpansion = true;
			
				string mName =  edbgMain.GridBoundColumns[e.ColIndex-2].MappingName;
				DataView dv = this.edbgMain.DataSource as DataView;

				if((sortHash[mName] is bool) && dv != null)
				{	
					if(!(bool)sortHash[mName])
					{
						dv.Sort = mName + " ASC";
						sortHash[mName] = true;
					}
					else
					{
						dv.Sort	= mName + " DESC";
						sortHash[mName] = false;
					}
				}
				DoExpansion();
				this.edbgMain.EndUpdate();
			}
		}


		private void OnTabChanged(object sender, System.EventArgs e)
		{
			bCheckSelections = false;
	
			this.edbgMain.BeginUpdate();

			if(this.tabExecutions.SelectedTab == this.tabMatch)
				topRow_U = this.edbgMain.TopRowIndex;	
			else
				topRow_M = this.edbgMain.TopRowIndex;

			DataView currentView = this.edbgMain.DataSource as DataView;
			if(currentView == null)
				return;
            
			if(this.tabExecutions.SelectedTab == this.tabMatch)
				currentView.RowFilter = GetFilter(currentView.RowFilter,true);
			else
				currentView.RowFilter = GetFilter(currentView.RowFilter,false);

			if(currentView == orderTable.DefaultView /*orderView*/)
				DoExpansion();

			if(this.tabExecutions.SelectedTab == this.tabMatch)
			{
				this.edbgMain.Selections.Clear();
				
				for(int x=0; x < gms_M.Count; x++)
				{
					this.edbgMain.Selections.Add(gms_M[x]);
				}

				this.edbgMain.TopRowIndex = topRow_M;

				if(gms_M.Count > 0)
					this.edbgMain.ScrollCellInView(gms_M[0]);
			}
			else
			{
				this.edbgMain.Selections.Clear();
				
				
				for(int x=0; x < gms_U.Count; x++)
				{
					this.edbgMain.Selections.Add(gms_U[x]);
				}

				this.edbgMain.TopRowIndex = topRow_U;

				if(gms_U.Count > 0)
					this.edbgMain.ScrollCellInView(gms_U[0]);

			}
			//			this.edbgMain.UpdateScrollBars();
			this.edbgMain.EndUpdate();

			bCheckSelections = true;
		}

		private void OnStatusTextChanged(object sender, System.EventArgs e)
		{
			toolTipExec.SetToolTip(labelStatus,labelStatus.Text);
		}
	
		private void onPrepareViewStyleInfo(object sender, Syncfusion.Windows.Forms.Grid.GridPrepareViewStyleInfoEventArgs e)
		{
			if(e.RowIndex != 0)
			{
				GridBoundRecordState gbrs = this.gridBinder.GetRecordStateAtRowIndex(e.RowIndex);
				if(gbrs!=null && gbrs.Table!=null && gbrs.Position>=0 && gbrs.Position<gbrs.Table.Count)
				{

					if(this.edbgMain.Selections.Ranges.AnyRangeContains(GridRangeInfo.Cell(e.RowIndex, e.ColIndex)) && !this.edbgMain.CurrentCell.IsEditing && colorsHash != null)
					{
						if(colorsHash[Nodes.Selection] is Hashtable)
						{
							Hashtable select = colorsHash[Nodes.Selection] as Hashtable;
							if(select != null)
							{
								e.Style.BackColor = (Color)select[Nodes.BackColor];
								e.Style.TextColor = (Color)select[Nodes.ForeColor];
								e.Style.Font = ToSynfusionFont(select[Nodes.Font] as Font);
								if(gbrs.LevelIndex == 1 && e.ColIndex == 2)
									e.Style.TextMargins.Left = 5;
							}
//							else
//							{
//								e.Style.BackColor = this.BackColor;
//								e.Style.TextColor = this.ForeColor;
//								e.Style.Font = ToSynfusionFont(this.Font);
//							}
						}
					}

					DataRowView drv = gbrs.Table[gbrs.Position] as DataRowView;

					if (drv != null)
					{		
						if(gbrs.LevelIndex == 1)
						{
							if(e.ColIndex == 2 && !this.edbgMain.Selections.Ranges.AnyRangeContains(GridRangeInfo.Cell(e.RowIndex, e.ColIndex)))
							{
								//e.Style.Text = "     " + e.Style.Text;
								e.Style.TextMargins.Left = 5;
							}

							if(drv.Row["ReportedOn"] == System.DBNull.Value)
								e.Style.Font.Italic = false;	
							else
							{
								e.Style.Font.Italic = true;
								if(e.ColIndex == 7)
									e.Style.CellTipText = drv.Row["ReportedOn"]+"";
							}
							if(e.ColIndex == 8)
								e.Style.CellTipText = drv.Row["OIRN"]+"";
						}
						else if (gbrs.LevelIndex == 0)
						{
							if((DataView)this.edbgMain.DataSource == orderTable.DefaultView /*orderView*/)
							{
								e.Style.Font.Bold = true;

								if(e.ColIndex == 7)
									e.Style.CellTipText = drv.Row["ReportedOn"]+"";
								if(e.ColIndex == 8)
									e.Style.CellTipText = drv.Row["OIRN"]+"";

								if(drv.Row["AllReported"]+"" == "true")
								{
									e.Style.Font.Italic = true;
								}
								else
								{
									e.Style.Font.Italic = false;
									//									if(bUnreportedOnly && e.ColIndex == 7)
									//										e.Style.Text = string.Empty;										
								}


							}
							else
							{
								if( drv.Row["ReportedOn"] == System.DBNull.Value)
									e.Style.Font.Italic = false;	
								else
								{
									e.Style.Font.Italic = true;
									if(e.ColIndex == 7)
										e.Style.CellTipText = drv.Row["ReportedOn"]+"";
								}
								if(e.ColIndex == 8)
									e.Style.CellTipText = drv.Row["OIRN"]+"";
							}
						}
					}
					
				}
			}

		}

		private void onGroupCheckChanged(object sender, System.EventArgs e)
		{
			GroupExecution = checkBoxGroup.Checked;		
		}

		private void OnUnreportedCheckChanged(object sender, System.EventArgs e)
		{
			UnreportedOnly = checkBoxUnreportedOnly.Checked;		
		}

		private void OnTabClick(object sender, System.EventArgs e)
		{
			this.edbgMain.BeginUpdate();
			this.edbgMain.Refresh();
			this.edbgMain.EndUpdate();
		}

		private void OnRowExpanded(object sender, Syncfusion.Windows.Forms.Grid.GridRowEventArgs e)
		{
			if(bCheckExpansion)
			{
				int rowIndex = e.RowIndex;
				GridBoundRecordState gbrs = this.gridBinder.GetRecordStateAtRowIndex(rowIndex);
				if(gbrs != null && gbrs.Table != null && gbrs.Position >= 0 && gbrs.Position < gbrs.Table.Count)
				{
					DataRowView drv = gbrs.Table[gbrs.Position] as DataRowView;

					if(drv != null)
					{
						DataView dv = this.edbgMain.DataSource as DataView;
						if(dv != null && dv == orderTable.DefaultView /*orderView*/)
							expansionHash[drv.Row["OrderID"]+""] = true;
					}
				}	
			}
		}

		private void OnRowCollapsed(object sender, Syncfusion.Windows.Forms.Grid.GridRowEventArgs e)
		{
			if(bCheckExpansion)
			{
				int rowIndex = e.RowIndex;
				GridBoundRecordState gbrs = this.gridBinder.GetRecordStateAtRowIndex(rowIndex);
				if(gbrs != null && gbrs.Table != null && gbrs.Position >= 0 && gbrs.Position < gbrs.Table.Count)
				{
					DataRowView drv = gbrs.Table[gbrs.Position] as DataRowView;

					if(drv != null)
					{
						DataView dv = this.edbgMain.DataSource as DataView;
						if(dv != null && dv == orderTable.DefaultView /*orderView*/)
							expansionHash[drv.Row["OrderID"]+""] = false;
					}
				}
			}
		}
		
		private void OnDrawCell(object sender, Syncfusion.Windows.Forms.Grid.GridDrawCellEventArgs e)
		{
			if( e.ColIndex == 1)
			{
				GridBoundRecordState state = this.edbgMain.Binder.GetRecordStateAtRowIndex(e.RowIndex);

				if( state.HasChildList && state.ChildCount == 0 && ((DataView)this.edbgMain.DataSource) == execTable.DefaultView /*execView*/)
				{
					int count;

					if (state.Position >= state.Table.Count)
						count = 0; //add row...
					else
					{
						this.edbgMain.BeginUpdate();
						this.edbgMain.Model.SuspendChangeEvents();
						count = state.ChildCount ;
						this.edbgMain.Model.ResumeChangeEvents();
						this.edbgMain.EndUpdate();
					}
					if(count == 0)
					{
						using(SolidBrush brush = new SolidBrush(e.Style.BackColor))
						{
							e.Graphics.FillRectangle(brush, e.Bounds);
							e.Cancel = true;
						}
					}
					
				}
			}
		}

		private void OnListChange(object sender, ListChangedEventArgs e)
		{
			if(e.ListChangedType == ListChangedType.Reset)
			{}
		}


		private void OnItemChange(object sender, ItemChangedEventArgs e)
		{
			this.edbgMain.Refresh();
		}

		private void OnGridSelChanged(object sender, GridSelectionChangedEventArgs e)
		{
			if(bCheckSelections)
			{
				if(this.tabExecutions.SelectedTab == this.tabMatch)
				{
					gms_M = this.edbgMain.Selections.GetSelectedRows(true,true);
					val_M = this.edbgMain.VScrollBar.Value;
				}
				else
				{
					gms_U = this.edbgMain.Selections.GetSelectedRows(true,true);
					val_U = this.edbgMain.VScrollBar.Value;
				}
			}
		}

	
		#endregion

		#region Live Pricing
		protected Icon iconLPConnected, iconLPDisconnected;
		protected void InitializeLP()
		{
			try
			{
				System.Reflection.Assembly myAssembly = System.Reflection.Assembly.GetExecutingAssembly();
				System.IO.Stream stream;
				stream = myAssembly.GetManifestResourceStream("XTreme.TI.ImagesAndIcons.LivePricing.green_light.ico");
				iconLPConnected = new Icon(stream);
				stream = myAssembly.GetManifestResourceStream("XTreme.TI.ImagesAndIcons.LivePricing.red_light.ico");
				iconLPDisconnected = new Icon(stream);
				stream.Close();
			}
			catch(Exception ex)
			{
	//			Logger.LogMessage("Mainform", "LP icons exception", LogEntryType.ERROR, -1, ex.ToString());
			}
		}
		protected void SetLPIconSafe(int val)
		{
			if(val>0)
				this.statusBarPanel1.Icon = this.iconLPConnected;
			else if(val<0)
				this.statusBarPanel1.Icon = this.iconLPDisconnected;
			else this.statusBarPanel1.Icon = null;

		}
		#endregion

		private void ExecutionListingForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			e.Cancel = true;
			this.Hide();
		}

		
	}

}